@@ -1,6 +1,6 @@
 #' @export
 `[.dmclist` <- function(x, i, ...) {
-  y = .subset(x, i)
-  attributes(y) = attributes(x)
+  y <- .subset(x, i)
+  attributes(y) <- attributes(x)
   return(y)
 }
